searchData={"items":[{"type":"module","doc":"`Supra` is a library of pipe-able functions and types for use with Ecto.","title":"Supra","ref":"Supra.html"},{"type":"module","doc":"Supra ships with multiple Elixir types for helping to document code with specs.\n\n- `t:Supra.change/1` - a type alias for `t:Ecto.Changeset.t/1`.\n- `t:Supra.result/1` - the result of an Ecto insert or update operation, where the\n  return value will be either `{:ok, t()}` or `{:error, Ecto.Changeset.t(t())}`.\n- `t:Supra.result/2` - the result of a function that will call ecto, and return either\n  `{:ok, t()}` or `{:error, error_t()}`. Useful for functions wrapping Ecto.Multi,\n  when the error changeset is different from the success schema.\n- `t:Supra.preloadable/1` - a type for a function that will pass arguments into\n  `c:Ecto.Repo.preload/3`, where the value may be a schema, nil, or a list of schemas.\n\n``` elixir\ndefmodule Core.Context do\n  alias Schema.Thing\n\n  @spec create_thing(Enum.t()) :: Supra.result(Thing.t())\n  def create_thing(attrs), do: new_thing(attrs) |> Core.Repo.insert()\n\n  @spec new_thing(Enum.t()) :: Supra.change(Thing.t())\n  def new_thing(attrs), do: Core.Context.Thing.changeset(attrs) |> Core.Repo.insert()\nend\n```","title":"Types - Supra","ref":"Supra.html#module-types"},{"type":"module","doc":"Supra includes several minimal wrappers for simple Ecto phrases, as well as a\ncomprehensive stream builder, allowing records to be streamed outside of a transaction,\nand with arbitrary ordering.\n\nSchema helpers:\n\n- `count/2` - a simple wrapper for `c:Ecto.Repo.aggregate/3`.\n- `first/2` - a pipe-able function for limiting query results.\n\nQuery helpers:\n\n- `format/3` - converts an `Ecto.Query` to SQL.\n- `limit/2` - a simple wrapper for `Ecto.Query.limit/3`.\n\nStream helpers:\n\n- `stream_by/3` - when a simple query should be streamed by a column that may be\n  uniquely identified by a single atom.\n- `stream/2` - when a more complex query should be streamed, where a `cursor_fun`\n  calls into the last returned value of a batch to find update a cursor, and a\n  `next_batch_fun` updates the query with a dynamic where clause to apply the\n  cursor to the next batch.","title":"Functions - Supra","ref":"Supra.html#module-functions"},{"type":"type","doc":"An alias for `t:Ecto.Changeset.t/1`","title":"Supra.change/1","ref":"Supra.html#t:change/1"},{"type":"function","doc":"Returns the number of rows in `queryable`","title":"Supra.count/2","ref":"Supra.html#count/2"},{"type":"function","doc":"Limits `queryable` to one result and returns that result","title":"Supra.first/2","ref":"Supra.html#first/2"},{"type":"function","doc":"Returns `query` as a string with all parameters formatted in the specified style. Styles are rendered via\n`IO.ANSI` which will only render stylized text if it thinks the output device can show them.\n\n* `:bright` renders parameters in brighter text, which is the most subtle of all the supported styles and may be hard\n  to differentiate.\n* `:color` renders parameters in different colors depending on their data types via `IO.ANSI.syntax_colors/0`.\n* `:inverse` (the default) renders parameters with an inverse background. This is quite visible and also compatible\n  when some other code such as a logger is colorizing the query.\n* `:underline` is a bit more subtle than `:inverse` but more visible than `:bright`.","title":"Supra.format/3","ref":"Supra.html#format/3"},{"type":"function","doc":"Applies `limit` to `queryable`","title":"Supra.limit/2","ref":"Supra.html#limit/2"},{"type":"type","doc":"An entity that will be passed into `c:Ecto.Repo.preload/3`","title":"Supra.preloadable/0","ref":"Supra.html#t:preloadable/0"},{"type":"type","doc":"An entity that will be passed into `c:Ecto.Repo.preload/3`","title":"Supra.preloadable/1","ref":"Supra.html#t:preloadable/1"},{"type":"type","doc":"The result of `c:Ecto.Repo.insert/2` or `c:Ecto.Repo.update/2`","title":"Supra.result/1","ref":"Supra.html#t:result/1"},{"type":"type","doc":"The result of `c:Ecto.Repo.insert/2` or `c:Ecto.Repo.update/2`, when the error changeset wraps a different schema","title":"Supra.result/2","ref":"Supra.html#t:result/2"},{"type":"function","doc":"Streams an Ecto query without requiring a transaction.","title":"Supra.stream/2","ref":"Supra.html#stream/2"},{"type":"function","doc":"- `batch_transform :: fun()` optional - An arity-1 function that receives each batch and transforms\n  it. May be used to manually run each batch through a set of preload functions. Note that, when\n  specified, the batch transform functions after executing the `:preload` option, but before the\n  `:cursor_fun`.\n- `cursor_fun :: fun()` required - An arity-1 function that will be given the last value returned\n  from the stream. This function will be evaluated to save the cursor value that will be used\n  to find the next batch.\n- `next_batch_fun :: fun()` required - An arity-1 function that will be given the cursor saved\n  from the previous batch. This function must return an `t:Ecto.Query.dynamic_expr/0` that may\n  used in a where clause to find the next batch.\n- `repo :: module()` required - An `Ecto.Repo` execute queries.\n- `batch_size :: integer()` default `100` - The size of batches to query from the database.\n- `preload :: term()` optional - An optional set of preloads to apply to each batch before\n  emitting members to the stream. This is preferred over query-time preloads for `has_many`\n  associations, as described below.","title":"Options - Supra.stream/2","ref":"Supra.html#stream/2-options"},{"type":"function","doc":"``` elixir\ndefmodule Core.Context do\n  alias Core.Context.Stuff\n  alias Core.Context.Thing\n  require Ecto.Query\n\n  @spec stream_things_by_email() :: Enum.t()\n  def stream_things_by_email do\n    Thing.Query.base()\n    |> Thing.Query.ordered_by_emails()\n    |> Supra.stream(\n      batch_size: 150,\n      cursor_fun: & &1.email,\n      next_batch_fun: fn last_email -> Ecto.Query.dynamic([things: t], t.email > ^last_email) end,\n      repo: Core.Repo\n    )\n  end\n\n  @spec stream_things_and_stuff() :: Enum.t()\n  def stream_things_and_stuff do\n    Thing.Query.base()\n    |> Thing.Query.ordered_by_name()\n    |> Supra.stream(\n      cursor_fun: & &1.name,\n      next_batch_fun: fn last_name -> Ecto.Query.dynamic([things: t], t.name > ^last_name) end,\n      preload: [stuff: Stuff.Query.unlost()],\n      repo: Core.Repo\n    )\n  end\nend\n```","title":"Examples - Supra.stream/2","ref":"Supra.html#stream/2-examples"},{"type":"function","doc":"When streaming a query with query-time preloads for `Ecto.Schema.has_many/3` associations, values may\nbe truncated for the last record of each stream batch. The `:preload` option to `Supra.stream/2` and\n`Supra.stream_by/3` operates on the returned values of each batch, and thus does not exhibit this problem.","title":"Warning - Supra.stream/2","ref":"Supra.html#stream/2-warning"},{"type":"function","doc":"Streams an Ecto query without requiring a transaction.\nMust be given the name of a non-nullable field to iterate over in batches.","title":"Supra.stream_by/3","ref":"Supra.html#stream_by/3"},{"type":"function","doc":"- `repo :: module()` required - An `Ecto.Repo` execute queries.\n- `batch_size :: integer()` default `100` - The size of batches to query from the database.\n- `order :: :asc | :desc` default `:asc` - The order in which to iterate over batches.\n- `preload :: term()` optional - An optional set of preloads to apply to each batch before\n  emitting members to the stream. This is preferred over query-time preloads for `has_many`\n  associations, as described below.","title":"Options - Supra.stream_by/3","ref":"Supra.html#stream_by/3-options"},{"type":"function","doc":"When streaming a query with query-time preloads for `Ecto.Schema.has_many/3` associations, values may\nbe truncated for the last record of each stream batch. The `:preload` option to `Supra.stream/2` and\n`Supra.stream_by/3` operates on the returned values of each batch, and thus does not exhibit this problem.","title":"Warning - Supra.stream_by/3","ref":"Supra.html#stream_by/3-warning"},{"type":"type","doc":"","title":"Supra.stream_by_opt/0","ref":"Supra.html#t:stream_by_opt/0"},{"type":"type","doc":"","title":"Supra.stream_by_opts/0","ref":"Supra.html#t:stream_by_opts/0"},{"type":"type","doc":"","title":"Supra.stream_opt/0","ref":"Supra.html#t:stream_opt/0"},{"type":"type","doc":"","title":"Supra.stream_opts/0","ref":"Supra.html#t:stream_opts/0"},{"type":"module","doc":"Internal module for formatting a sql query","title":"Supra.Format","ref":"Supra.Format.html"},{"type":"function","doc":"See `Supra.format/3`","title":"Supra.Format.format/2","ref":"Supra.Format.html#format/2"},{"type":"type","doc":"","title":"Supra.Format.style/0","ref":"Supra.Format.html#t:style/0"},{"type":"exception","doc":"","title":"Supra.Error","ref":"Supra.Error.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.2"}}